import os

import pytest
from path import Path

from . import initialize_git_repo_and_commit, prepare_project

DIST_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "../dist"))

# Test if package can be imported to allow testing on
# conda-forge where ``pytest-virtualenv`` is not available.
pytest.importorskip("pytest_virtualenv", reason="pytest_virtualenv not available. See #228")


def test_source_distribution(virtualenv):
    sdists = Path(DIST_DIR).files(match="*.tar.gz") if Path(DIST_DIR).exists() else []
    if not sdists:
        pytest.skip("no source distribution available")
    assert len(sdists) == 1

    virtualenv.run("pip install %s" % sdists[0])
    assert "scikit-build" in virtualenv.installed_packages()

    prepare_project("hello-no-language", virtualenv.workspace, force=True)
    initialize_git_repo_and_commit(virtualenv.workspace, verbose=False)

    virtualenv.run("python setup.py bdist_wheel")


def test_wheel(virtualenv):
    wheels = Path(DIST_DIR).files(match="*.whl") if Path(DIST_DIR).exists() else []
    if not wheels:
        pytest.skip("no wheel available")
    assert len(wheels) == 1

    virtualenv.run("pip install %s" % wheels[0])
    assert "scikit-build" in virtualenv.installed_packages()

    prepare_project("hello-no-language", virtualenv.workspace, force=True)
    initialize_git_repo_and_commit(virtualenv.workspace, verbose=False)

    virtualenv.run("python setup.py bdist_wheel")



from skbuild import setup

setup(
    name="hello",
    version="1.2.3",
    description=(
        "a minimal example packagze that should always fail to build "
        "because it provides a _hello.cxx with a compile error"
    ),
    author="The scikit-build team",
    license="MIT",
    packages=["hello"],
)



"""This module defines object specific to Cygwin platform."""

import sys
import textwrap

from . import abstract
from .abstract import CMakeGenerator


# pylint:disable=abstract-method
class CygwinPlatform(abstract.CMakePlatform):
    """Cygwin implementation of :class:`.abstract.CMakePlatform`."""

    def __init__(self):
        super(CygwinPlatform, self).__init__()
        self.default_generators = [CMakeGenerator("Ninja"), CMakeGenerator("Unix Makefiles")]

    @property
    def generator_installation_help(self):
        """Return message guiding the user for installing a valid toolchain."""
        return (
            textwrap.dedent(
                """
            Building Cygwin wheels for Python {pyver} requires Cygwin packages
            ninja or make and compilers from e.g. gcc-core and gcc-g++.
            Get them here:

              https://cygwin.com/packages/package_list.html
            """
            )
            .format(pyver="%s.%s" % sys.version_info[:2])
            .strip()
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""test_skbuild
----------------------------------

Tests for `skbuild` module.
"""

import os
import platform
import sys

import pytest

from skbuild.constants import CMAKE_BUILD_DIR
from skbuild.exceptions import SKBuildError
from skbuild.platform_specifics import get_platform
from skbuild.platform_specifics.windows import VS_YEAR_TO_VERSION, find_visual_studio

from . import get_cmakecache_variables, project_setup_py_test, push_dir, push_env, which


def test_generator_selection():
    version = sys.version_info
    env_generator = os.environ.get("CMAKE_GENERATOR")
    this_platform = platform.system().lower()
    get_best_generator = get_platform().get_best_generator
    arch = platform.architecture()[0]

    if env_generator:
        assert get_best_generator(env_generator).name == env_generator

    if this_platform == "windows":
        # assert that we are running a supported version of python
        py_27_32 = (version.major == 2 and version.minor >= 7) or (version.major == 3 and version.minor <= 2)

        py_33_34 = version.major == 3 and (3 <= version.minor <= 4)

        py_35 = version.major == 3 and version.minor >= 5

        assert len(tuple(filter(bool, (py_27_32, py_33_34, py_35)))) == 1

        # Expected Visual Studio version
        if py_27_32:
            vs_generator = "Visual Studio 9 2008"
            vs_version = 9
        elif py_33_34:
            vs_generator = "Visual Studio 10 2010"
            vs_version = 10
        else:
            vs_generator = "Visual Studio 14 2015"
            vs_version = 14
        vs_generator += " Win64" if arch == "64bit" else ""

        has_vs_2017 = find_visual_studio(vs_version=VS_YEAR_TO_VERSION["2017"])
        has_vs_2019 = find_visual_studio(vs_version=VS_YEAR_TO_VERSION["2019"])
        has_vs_2022 = find_visual_studio(vs_version=VS_YEAR_TO_VERSION["2022"])

        # Apply to VS <= 14 (2015)
        has_vs_ide_vcvars = any(
            [
                os.path.exists(path_pattern % vs_version)
                for path_pattern in ["C:/Program Files (x86)/Microsoft Visual Studio %.1f/VC/vcvarsall.bat"]
            ]
        )

        # As of Dec 2016, this is available only for VS 9.0
        has_vs_for_python_vcvars = any(
            [
                os.path.exists(os.path.expanduser(path_pattern % vs_version))
                for path_pattern in [
                    "~/AppData/Local/Programs/Common/Microsoft/Visual C++ for Python/%.1f/vcvarsall.bat",
                    "C:/Program Files (x86)/Common Files/Microsoft/Visual C++ for Python/%.1f/vcvarsall.bat",
                ]
            ]
        )

        # If environment exists, update the expected generator
        if (has_vs_for_python_vcvars or has_vs_ide_vcvars) and which("ninja.exe"):
            assert get_best_generator().name == "Ninja"

        elif has_vs_2017:
            vs_generator = "Visual Studio 15 2017"
            # Early versions of 2017 may not ship with Ninja (TODO: check)
            assert get_best_generator().name in {"Ninja", vs_generator}

        elif has_vs_2019 or has_vs_2022:
            # ninja is provided by the CMake extension bundled with Visual Studio 2017
            # C:/Program Files (x86)/Microsoft Visual Studio/2017/Professional/Common7/IDE/CommonExtensions/Microsoft/CMake/Ninja/ninja.exe  # noqa: E501
            assert get_best_generator().name == "Ninja"

        elif has_vs_ide_vcvars:
            assert get_best_generator().name == vs_generator

        elif has_vs_for_python_vcvars:
            assert get_best_generator().name == "NMake Makefiles"

    elif this_platform in ["darwin", "linux"]:
        generator = "Ninja" if which("ninja") else "Unix Makefiles"
        assert get_best_generator().name == generator


@pytest.mark.parametrize("generator, expected_make_program", [("NMake Makefiles", "nmake"), ("Unix Makefiles", "make")])
def test_generator(generator, expected_make_program):

    generator_platform = {"NMake Makefiles": ["windows"], "Unix Makefiles": ["darwin", "linux"]}
    assert generator in generator_platform

    this_platform = platform.system().lower()
    if this_platform not in generator_platform[generator]:
        pytest.skip("{} generator is available only on {}".format(generator, this_platform.title()))

    @project_setup_py_test("hello-cpp", ["build"])
    def run_build():
        pass

    with push_env(CMAKE_GENERATOR=generator):
        tmp_dir = run_build()[0]
        cmakecache = tmp_dir.join(CMAKE_BUILD_DIR()).join("CMakeCache.txt")
        assert cmakecache.exists()
        variables = get_cmakecache_variables(str(cmakecache))
        make_program = variables["CMAKE_MAKE_PROGRAM"][1] if "CMAKE_MAKE_PROGRAM" in variables else ""
        assert make_program.endswith(expected_make_program) or make_program.endswith("%s.exe" % expected_make_program)


@pytest.mark.parametrize(
    "generator_args",
    [
        ["-G", "invalid"],
        ["--", "-G", "invalid"],
    ],
)
def test_invalid_generator(generator_args):
    with push_dir():

        build_args = ["build"]
        build_args.extend(generator_args)

        @project_setup_py_test("hello-no-language", build_args, disable_languages_test=True)
        def run():
            pass

        failed = False
        message = ""
        try:
            run()
        except SystemExit as e:
            failed = isinstance(e.code, SKBuildError)
            message = str(e)

        assert failed
        assert "scikit-build could not get a working generator " "for your system. Aborting build." in message


@pytest.mark.skipif(sys.platform != "win32", reason="Requires Windows")
@pytest.mark.parametrize("vs_year", ["2008", "2010", "2012", "2013", "2015", "2017", "2019", "2022"])
def test_platform_windows_find_visual_studio(vs_year):
    """If the environment variable ``SKBUILD_TEST_FIND_VS<vs_year>_INSTALLATION_EXPECTED`` is set,
    this test asserts the value returned by :func:`skbuild.platforms.windows.find_visual_studio()`.
    It skips the test otherwise.

    Setting the environment variable to 1 means that the corresponding Visual Studio version
    is expected to be installed. Setting it to 0, means otherwise.
    """
    env_var = "SKBUILD_TEST_FIND_VS%s_INSTALLATION_EXPECTED" % vs_year
    if env_var not in os.environ:
        pytest.skip("env. variable %s is not set" % env_var)

    valid_path_expected = bool(int(os.environ[env_var]))
    vs_path = find_visual_studio(VS_YEAR_TO_VERSION[vs_year])
    if valid_path_expected:
        assert os.path.exists(vs_path)
    else:
        assert vs_path == ""


@pytest.mark.skipif(sys.version_info < (3, 5), reason="Python 3.5+ required on Windows")
@pytest.mark.skipif(sys.platform != "win32", reason="Requires Windows")
def test_toolset():
    py_35 = sys.version_info[:2] == (3, 5)

    has_vs_2017 = find_visual_studio(vs_version=VS_YEAR_TO_VERSION["2017"])
    if not has_vs_2017:
        pytest.skip("Visual Studio 15 2017 is not found")

    arch = platform.architecture()[0]
    vs_generator = "Visual Studio 15 2017"
    orig_generator = vs_generator
    if arch == "64bit":
        vs_generator += " Win64"

    @project_setup_py_test("hello-cpp", ["build", "-G", vs_generator])
    def run_build():
        pass

    tmp_dir = run_build()[0]

    cmakecache = tmp_dir.join(CMAKE_BUILD_DIR()).join("CMakeCache.txt")
    variables = get_cmakecache_variables(str(cmakecache))

    generator = variables["CMAKE_GENERATOR"][1]
    assert generator == orig_generator

    var_toolset = variables["CMAKE_GENERATOR_TOOLSET"]
    toolset = var_toolset[1]

    if py_35:
        assert toolset == "v140"
    else:
        assert toolset == "v141"



from skbuild import setup

setup(
    name="fail_with_syntax_error_cmakelists",
    version="0.0.1",
    description=(
        "test project that should always fail to build because it " "provides a CMakeLists.txt with a syntax error"
    ),
    author="The scikit-build team",
    license="MIT",
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""test_cmaker
----------------------------------

Tests for CMaker functionality.
"""

import os
import re
import sys
import textwrap

import pytest

from skbuild.cmaker import CMaker, has_cmake_cache_arg
from skbuild.constants import (
    CMAKE_BUILD_DIR,
    CMAKE_DEFAULT_EXECUTABLE,
    CMAKE_INSTALL_DIR,
)
from skbuild.exceptions import SKBuildError
from skbuild.utils import push_dir, to_unix_path

from . import _tmpdir, get_cmakecache_variables


def test_get_python_version():
    assert re.match(r"^[23](\.?)\d+$", CMaker.get_python_version())


def test_get_python_include_dir():
    python_include_dir = CMaker.get_python_include_dir(CMaker.get_python_version())
    assert python_include_dir
    assert os.path.exists(python_include_dir)


def test_get_python_library():
    python_library = CMaker.get_python_library(CMaker.get_python_version())
    assert python_library
    assert os.path.exists(python_library)


def test_cmake_executable():
    assert CMaker().cmake_executable == CMAKE_DEFAULT_EXECUTABLE


def test_has_cmake_cache_arg():
    cmake_args = ["-DFOO:STRING=42", "-DBAR", "-DCLIMBING:BOOL=ON"]
    assert has_cmake_cache_arg(cmake_args, "FOO", "42")
    assert not has_cmake_cache_arg(cmake_args, "foo", "42")
    assert not has_cmake_cache_arg(cmake_args, "FOO", "43")
    assert not has_cmake_cache_arg(cmake_args, "BAR")
    assert not has_cmake_cache_arg(cmake_args, "BA")
    assert not has_cmake_cache_arg(cmake_args, "BAR", None)
    assert not has_cmake_cache_arg(cmake_args, "BAR", "42")
    assert has_cmake_cache_arg(cmake_args, "CLIMBING")
    assert has_cmake_cache_arg(cmake_args, "CLIMBING", None)
    assert has_cmake_cache_arg(cmake_args, "CLIMBING", "ON")

    override = ["-DOTHER:STRING=C", "-DOVERRIDE:STRING=A", "-DOVERRIDE:STRING=B"]
    assert has_cmake_cache_arg(override, "OVERRIDE")
    assert has_cmake_cache_arg(override, "OVERRIDE", "B")
    assert not has_cmake_cache_arg(override, "OVERRIDE", "A")
    # ensure overriding doesn't magically have side effects.
    assert has_cmake_cache_arg(override, "OTHER")
    assert has_cmake_cache_arg(override, "OTHER", "C")
    assert not has_cmake_cache_arg(override, "OTHER", "A")
    assert not has_cmake_cache_arg(override, "OTHER", "B")


def test_make_without_build_dir_fails():
    src_dir = _tmpdir("test_make_without_build_dir_fails")
    with push_dir(str(src_dir)), pytest.raises(SKBuildError) as excinfo:
        CMaker().make()
    assert "Did you forget to run configure before make" in str(excinfo.value)


def test_make_without_configure_fails(capfd):
    src_dir = _tmpdir("test_make_without_configure_fails")
    src_dir.ensure(CMAKE_BUILD_DIR(), dir=1)
    with push_dir(str(src_dir)), pytest.raises(SKBuildError) as excinfo:
        CMaker().make()
    _, err = capfd.readouterr()
    assert "An error occurred while building with CMake." in str(excinfo.value)
    assert "Error: could not load cache" in err


@pytest.mark.parametrize("configure_with_cmake_source_dir", (True, False))
def test_make(configure_with_cmake_source_dir, capfd):
    tmp_dir = _tmpdir("test_make")
    with push_dir(str(tmp_dir)):

        src_dir = tmp_dir.ensure("SRC", dir=1)
        src_dir.join("CMakeLists.txt").write(
            textwrap.dedent(
                """
            cmake_minimum_required(VERSION 3.5.0)
            project(foobar NONE)
            file(WRITE "${CMAKE_BINARY_DIR}/foo.txt" "# foo")
            install(FILES "${CMAKE_BINARY_DIR}/foo.txt" DESTINATION ".")
            install(CODE "message(STATUS \\"Project has been installed\\")")
            message(STATUS "CMAKE_SOURCE_DIR:${CMAKE_SOURCE_DIR}")
            message(STATUS "CMAKE_BINARY_DIR:${CMAKE_BINARY_DIR}")
            """
            )
        )
        src_dir.ensure(CMAKE_BUILD_DIR(), dir=1)

        with push_dir(str(src_dir) if not configure_with_cmake_source_dir else str(tmp_dir.ensure("BUILD", dir=1))):
            cmkr = CMaker()
            config_kwargs = {}
            if configure_with_cmake_source_dir:
                config_kwargs["cmake_source_dir"] = str(src_dir)
            env = cmkr.configure(**config_kwargs)
            cmkr.make(env=env)

        messages = ["Project has been installed"]

        if configure_with_cmake_source_dir:
            messages += [
                "/SRC",
                "/BUILD/{}".format(to_unix_path(CMAKE_BUILD_DIR())),
                "/BUILD/{}/./foo.txt".format(to_unix_path(CMAKE_INSTALL_DIR())),
            ]
        else:
            messages += [
                "/SRC",
                "/SRC/{}".format(to_unix_path(CMAKE_BUILD_DIR())),
                "/SRC/{}/./foo.txt".format(to_unix_path(CMAKE_INSTALL_DIR())),
            ]

        out, _ = capfd.readouterr()
        for message in messages:
            assert message in out


@pytest.mark.parametrize("install_target", ("", "install", "install-runtime", "nonexistant-install-target"))
def test_make_with_install_target(install_target, capfd):
    tmp_dir = _tmpdir("test_make_with_install_target")
    with push_dir(str(tmp_dir)):

        tmp_dir.join("CMakeLists.txt").write(
            textwrap.dedent(
                """
            cmake_minimum_required(VERSION 3.5.0)
            project(foobar NONE)
            file(WRITE "${CMAKE_BINARY_DIR}/foo.txt" "# foo")
            file(WRITE "${CMAKE_BINARY_DIR}/runtime.txt" "# runtime")
            install(FILES "${CMAKE_BINARY_DIR}/foo.txt" DESTINATION ".")
            install(CODE "message(STATUS \\"Project has been installed\\")")
            install(FILES "${CMAKE_BINARY_DIR}/runtime.txt" DESTINATION "." COMPONENT runtime)
            install(CODE "message(STATUS \\"Runtime component has been installed\\")" COMPONENT runtime)

            # Add custom target to only install component: runtime (libraries)
            add_custom_target(install-runtime
              ${CMAKE_COMMAND}
              -DCMAKE_INSTALL_COMPONENT=runtime
              -P "${PROJECT_BINARY_DIR}/cmake_install.cmake"
              )
            """
            )
        )

        with push_dir(str(tmp_dir)):
            cmkr = CMaker()
            env = cmkr.configure()
            if install_target in ["", "install", "install-runtime"]:
                cmkr.make(install_target=install_target, env=env)
            else:
                with pytest.raises(SKBuildError) as excinfo:
                    cmkr.make(install_target=install_target, env=env)
                assert "check the install target is valid" in str(excinfo.value)

        out, err = capfd.readouterr()
        # This message appears with both install_targets: default 'install' and
        # 'install-runtime'
        message = "Runtime component has been installed"
        if install_target in ["install", "install-runtime"]:
            assert message in out

        # One of these error appears with install_target: nonexistant-install-target
        err_message1 = "No rule to make target"
        err_message2 = "unknown target"
        err_message3 = "CMAKE_MAKE_PROGRAM is not set"  # Showing up on windows-2016
        if install_target == "nonexistant-install-target":
            assert err_message1 in err or err_message2 in err or err_message3 in err


def test_configure_with_cmake_args(capfd):
    tmp_dir = _tmpdir("test_configure_with_cmake_args")
    with push_dir(str(tmp_dir)):
        unused_vars = [
            "CMAKE_EXPECTED_BAR",
            "CMAKE_EXPECTED_FOO",
            "PYTHON_VERSION_STRING",
            "SKBUILD",
        ]

        find_python_prefixes = [
            "Python{}".format(sys.version_info[0]),
            "Python",
            "PYTHON",
        ]

        for prefix in find_python_prefixes:
            unused_vars.extend(
                [
                    (prefix + "_EXECUTABLE"),
                    (prefix + "_INCLUDE_DIR"),
                    (prefix + "_LIBRARY"),
                ]
            )

            try:
                import numpy as np  # noqa: F401

                unused_vars.append(prefix + "_NumPy_INCLUDE_DIRS")
            except ImportError:
                pass

        tmp_dir.join("CMakeLists.txt").write(
            textwrap.dedent(
                """
            cmake_minimum_required(VERSION 3.5.0)
            project(foobar NONE)
            # Do not complain about missing arguments passed to the main
            # project
            foreach(unused_var IN ITEMS
            {}
              )
            endforeach()
            """
            ).format("\n".join("  ${{{}}}".format(unused) for unused in unused_vars))
        )

        with push_dir(str(tmp_dir)):
            cmkr = CMaker()
            cmkr.configure(clargs=["-DCMAKE_EXPECTED_FOO:STRING=foo", "-DCMAKE_EXPECTED_BAR:STRING=bar"], cleanup=False)

        cmakecache = tmp_dir.join("_cmake_test_compile", "build", "CMakeCache.txt")
        assert cmakecache.exists()
        variables = get_cmakecache_variables(str(cmakecache))
        assert variables.get("CMAKE_EXPECTED_FOO", (None, None))[1] == "foo"
        assert variables.get("CMAKE_EXPECTED_BAR", (None, None))[1] == "bar"

        unexpected = "Manually-specified variables were not used by the project"
        _, err = capfd.readouterr()
        assert unexpected not in err


def test_check_for_bad_installs(tmpdir):
    with push_dir(str(tmpdir)):
        tmpdir.ensure(CMAKE_BUILD_DIR(), "cmake_install.cmake").write(
            textwrap.dedent(
                """
            file(INSTALL DESTINATION "${CMAKE_INSTALL_PREFIX}/../hello" TYPE FILE FILES "/path/to/hello/world.py")
            """
            )
        )
        with pytest.raises(SKBuildError) as excinfo:
            CMaker.check_for_bad_installs()
        assert "CMake-installed files must be within the project root" in str(excinfo.value)



"""This module defines custom implementation of ``sdist`` setuptools command."""

from setuptools.command.sdist import sdist as _sdist

from ..utils import distribution_hide_listing, distutils_log, new_style
from . import set_build_base_mixin


class sdist(set_build_base_mixin, new_style(_sdist)):
    """Custom implementation of ``sdist`` setuptools command."""

    def make_release_tree(self, base_dir, files):
        """Handle --hide-listing option."""
        with distribution_hide_listing(self.distribution):
            super(sdist, self).make_release_tree(base_dir, files)
        distutils_log.info("copied %d files", len(files))

    # pylint:disable=too-many-arguments, unused-argument
    def make_archive(self, base_name, _format, root_dir=None, base_dir=None, owner=None, group=None):
        """Handle --hide-listing option."""
        distutils_log.info("creating '%s' %s archive and adding '%s' to it", base_name, _format, base_dir)
        with distribution_hide_listing(self.distribution):
            super(sdist, self).make_archive(base_name, _format, root_dir, base_dir)

    def run(self, *args, **kwargs):
        """Force :class:`.egg_info.egg_info` command to run."""
        self.run_command("generate_source_manifest")
        super(sdist, self).run(*args, **kwargs)



"""
This module defines constants commonly used in scikit-build.
"""

import os
import platform
import sys
from distutils.util import get_platform

CMAKE_DEFAULT_EXECUTABLE = "cmake"
"""Default path to CMake executable."""


def _default_skbuild_plat_name():
    """Get default platform name.

    On linux and windows, it corresponds to :func:`distutils.util.get_platform()`.

    On macOS, it corresponds to the version and machine associated with :func:`platform.mac_ver()`.
    """
    if sys.platform != "darwin":
        return get_platform()

    supported_macos_architectures = {"x86_64", "arm64"}
    macos_universal2_architectures = {"x86_64", "arm64"}

    # See https://github.com/scikit-build/scikit-build/issues/643 for a weird cross
    # compiling bug that forces us to avoid getting machine from platform.mac_ver()[2]
    release = platform.mac_ver()[0]
    machine = platform.machine()

    # If the MACOSX_DEPLOYMENT_TARGET environment variable is defined, use
    # it, as it will be the most accurate. Otherwise use the value returned by
    # platform.mac_ver() provided by the platform module available in the
    # Python standard library.
    #
    # Note that on macOS, distutils.util.get_platform() is not used because
    # it returns the macOS version on which Python was built which may be
    # significantly older than the user's current machine.
    release = os.environ.get("MACOSX_DEPLOYMENT_TARGET", release)
    major_macos, minor_macos = release.split(".")[:2]

    # On macOS 11+, only the major version matters.
    if int(major_macos) >= 11:
        minor_macos = "0"

    # Use CMAKE_OSX_ARCHITECTURES if that is set, otherwise use ARCHFLAGS,
    # which is the variable used by Setuptools. Fall back to the machine arch
    # if neither of those is given.

    archflags = os.environ.get("ARCHFLAGS")
    if archflags is not None:
        machine = ";".join(set(archflags.split()) & supported_macos_architectures)

    machine = os.environ.get("CMAKE_OSX_ARCHITECTURES", machine)

    # Handle universal2 wheels, if those two architectures are requested.
    if set(machine.split(";")) == macos_universal2_architectures:
        machine = "universal2"

    return "macosx-{}.{}-{}".format(major_macos, minor_macos, machine)


_SKBUILD_PLAT_NAME = _default_skbuild_plat_name()


def set_skbuild_plat_name(plat_name):
    """Set platform name associated with scikit-build functions returning a path:

    * :func:`SKBUILD_DIR()`
    * :func:`SKBUILD_MARKER_FILE()`
    * :func:`CMAKE_BUILD_DIR()`
    * :func:`CMAKE_INSTALL_DIR()`
    * :func:`CMAKE_SPEC_FILE()`
    * :func:`SETUPTOOLS_INSTALL_DIR()`
    """
    global _SKBUILD_PLAT_NAME  # pylint: disable=global-statement
    _SKBUILD_PLAT_NAME = plat_name


def skbuild_plat_name():
    """Get platform name formatted as `<operating_system>[-<operating_system_version>]-<machine_architecture>`.

    Default value corresponds to :func:`_default_skbuild_plat_name()` and can be overridden
    with :func:`set_skbuild_plat_name()`.

    Examples of values are `macosx-10.9-x86_64`, `linux-x86_64`, `linux-i686` or `win-am64`.
    """
    return _SKBUILD_PLAT_NAME


def SKBUILD_DIR():
    """Top-level directory where setuptools and CMake directories are generated."""
    return os.path.join(
        "_skbuild",
        "{}-{}".format(_SKBUILD_PLAT_NAME, ".".join(map(str, sys.version_info[:2]))),
    )


def SKBUILD_MARKER_FILE():
    """Marker file used by :func:`skbuild.command.generate_source_manifest.generate_source_manifest.run()`."""
    return os.path.join(SKBUILD_DIR(), "_skbuild_MANIFEST")


def CMAKE_BUILD_DIR():
    """CMake build directory."""
    return os.path.join(SKBUILD_DIR(), "cmake-build")


def CMAKE_INSTALL_DIR():
    """CMake install directory."""
    return os.path.join(SKBUILD_DIR(), "cmake-install")


def CMAKE_SPEC_FILE():
    """CMake specification file storing CMake version, CMake configuration arguments and
    environment variables ``PYTHONNOUSERSITE`` and ``PYTHONPATH``.
    """
    return os.path.join(CMAKE_BUILD_DIR(), "CMakeSpec.json")


def SETUPTOOLS_INSTALL_DIR():
    """Setuptools install directory."""
    return os.path.join(SKBUILD_DIR(), "setuptools")



from skbuild import setup

setup(
    name="fail_outside_project_root",
    version="0.0.1",
    description=(
        "test project that should always fail to build because it "
        "tries to CMake-install something outside of its root"
    ),
    author="The scikit-build team",
    license="MIT",
)



"""This module defines object specific to BSD platform."""

from . import unix


# pylint:disable=abstract-method
class BSDPlatform(unix.UnixPlatform):
    """BSD implementation of :class:`.abstract.CMakePlatform`."""
